// System.Zip prior to XE3 (prob XE or XE2)
 
procedure TZipFile.Extract(Index: Integer; Path: string; CreateSubdirs: Boolean);
var
  LInStream, LOutStream: TStream;
  LHeader: TZipHeader;
  LDir, LFileName: string;
  Bytes: array [0..4095] of Byte;
  ReadBytes: Int64;
begin
  // Get decompression stream for file
  Read(Index, LInStream, LHeader);
  try
    LFileName := string(FFiles[Index].FileName);
{$IFDEF MSWINDOWS} // ZIP stores files with '/', so translate to a relative Windows path.
    LFileName := StringReplace(LFileName, '/', '\', [rfReplaceAll]);
{$ENDIF}
    // CreateSubDirs = False assumes the user passed in the path where they want the file to end up
    if CreateSubdirs then
      LFileName := TPath.Combine(Path, LFileName)
    else
      LFileName := TPath.Combine(Path, ExtractFileName(LFileName));
    // Force directory creation
    LDir := ExtractFileDir(LFileName);
    if CreateSubdirs and (LDir <> '') then
      TDirectory.CreateDirectory(ExtractFileDir(LFileName));
    // Open the File For output
    if LFileName[Length(LFileName)] = PathDelim then
      Exit; // Central Directory Entry points at a directory, not a file.
    LOutStream := TFileStream.Create(LFileName, fmCreate);
    try // And Copy from the decompression stream.
      if (LHeader.Flag and (1 SHL 2)) = 0 then                                        //////////// 1 SHL 3 changed to 1 SHL 2 as per QC issue 99443 suggestion ///////////
        if FFiles[Index].UncompressedSize > 0 then // Special case for empty files.
          LOutStream.CopyFrom(LInStream, FFiles[Index].UncompressedSize)
        else
        begin
          //CRC, Uncompressed, and Compressed Size follow the compressed data.
          while True do
          begin
            ReadBytes := LInStream.Read(Bytes, Length(Bytes));
            LOutStream.Write(Bytes, ReadBytes);
            if ReadBytes < Length(Bytes) then
              break;
          end;
        end;
    finally
      LOutStream.Free;
    end;
  finally
    LInStream.Free;
  end;
end;
 